var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "fs", "path"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.Languages = void 0;
    const fs_1 = __importDefault(require("fs"));
    const path_1 = __importDefault(require("path"));
    var Languages;
    (function (Languages) {
        Languages["bg"] = "bg";
        Languages["cs"] = "cs";
        Languages["de"] = "de";
        Languages["en_gb"] = "en-gb";
        Languages["es"] = "es";
        Languages["fr"] = "fr";
        Languages["hu"] = "hu";
        Languages["id"] = "id";
        Languages["it"] = "it";
        Languages["ja"] = "ja";
        Languages["ko"] = "ko";
        Languages["nl"] = "nl";
        Languages["pl"] = "pl";
        Languages["ps"] = "ps";
        Languages["pt_br"] = "pt-br";
        Languages["ru"] = "ru";
        Languages["tr"] = "tr";
        Languages["uk"] = "uk";
        Languages["zh_hans"] = "zh-hans";
        Languages["zh_hant"] = "zh-hant";
    })(Languages = exports.Languages || (exports.Languages = {}));
    /**
     * 使用了monaco-editor-nls的语言映射包，把原始localize(data, message)的方法，替换成了localize(path, data, defaultMessage)
     * @param options 替换语言包
     * @returns
     */
    function default_1(options = { locale: Languages.es }) {
        return {
            name: 'rollup-plugin-monaco-editor-nls',
            enforce: 'pre',
            /** 处理vite在esbuild情况下预编译生成chunk找不到nls替换文件 */
            config: (config) => {
                config.optimizeDeps = config.optimizeDeps
                    ? {
                        ...config.optimizeDeps,
                        exclude: config.optimizeDeps.exclude
                            ? [...config.optimizeDeps.exclude, `monaco-editor`]
                            : [`monaco-editor`],
                    }
                    : {
                        exclude: [`monaco-editor`],
                    };
                return config;
            },
            load(filepath) {
                if (/esm\/vs\/nls\.js/.test(filepath)) {
                    const code = getLocalizeCode(options.locale);
                    return code;
                }
            },
            transform(code, filepath) {
                if (/monaco-editor[\\\/]esm[\\\/]vs.+\.js/.test(filepath) &&
                    !/esm\/vs\/.*nls\.js/.test(filepath)) {
                    const re = /(?:monaco-editor\/esm\/)(.+)(?=\.js)/;
                    if (re.exec(filepath)) {
                        const path = RegExp.$1;
                        code = code.replace(/localize\(/g, `localize('${path}', `);
                        return {
                            code: code,
                        };
                    }
                }
                return {
                    code: code,
                };
            },
        };
    }
    exports.default = default_1;
    function getLocalizeCode(locale) {
        const locale_data_path = path_1.default.join(__dirname, `./locale/${locale}.json`);
        const CURRENT_LOCALE_DATA = fs_1.default.readFileSync(locale_data_path);
        return `
        function _format(message, args) {
            var result;
            if (args.length === 0) {
                result = message;
            } else {
                result = String(message).replace(/\{(\d+)\}/g, function (match, rest) {
                    var index = rest[0];
                    return typeof args[index] !== 'undefined' ? args[index] : match;
                });
            }
            return result;
        }

        export function localize(path, data, defaultMessage) {
            var key = typeof data === 'object' ? data.key : data;
            var data = ${CURRENT_LOCALE_DATA} || {};
            var message = (data[path] || {})[key];
            if (!message) {
                message = defaultMessage;
            }
            var args = [];
            for (var _i = 3; _i < arguments.length; _i++) {
                args[_i - 3] = arguments[_i];
            }
            return _format(message, args);
        }
    `;
    }
});
